# -*- coding: utf-8 -*-
import os
import warnings

from .helpers import dotenv_to_dict, envar_exists, get_envar


class ConfigVar:
    def __init__(self, name, value=None, alias=None, default=None, use_env=False):
        self.name = name
        self._value = value
        self.alias = alias
        self.default = default
        self.use_env = use_env

    @property
    def value(self):
        if self.use_env:
            if envar_exists(self.name):
                return get_envar(self.name)
            elif not envar_exists(self.name) and self._value:
                return self._value
            elif not envar_exists(self.name) and self.default:
                return self.default
            elif not envar_exists(self.name):
                raise KeyError(f"{self.name} is not a valid value.")
        return self._value

    @value.setter
    def value(self, value):
        self._value = value

    def is_renamed(self):
        return self.alias is not None


class ConfigVars:
    def __init__(self):
        self.store = {}

    def add(self, name, **kwargs):
        if self.exists(name):
            for key, value in kwargs.items():
                if value:
                    setattr(self.store[name], key, value)
        else:
            self.store[name] = ConfigVar(name, **kwargs)

    def get(self, name):
        try:
            return self.store[name]
        except KeyError:
            for _, var in self.store.items():
                if var.is_renamed:
                    return var

    def exists(self, name):
        return name in self.store.keys()

    def to_dict(self):
        rv = {}
        for key in self.store.keys():
            rv[key] = self.get(key).value
        return rv


class Config(dict):
    def __init__(self):
        self.config_vars = ConfigVars()

    def __contains__(self, key):
        return self.config_vars.exists(key)

    def __setitem__(self, key, value):
        self.config_vars.add(key, value=value)

    def __getitem__(self, key):
        var = self.config_vars.get(key)
        if var.alias and var.alias != key:
            raise KeyError(f"{key} is not a valid key.")
        return var.value

    def __str__(self):
        return str(self.config_vars.to_dict())

    def from_object(self, obj: object):
        """Read configuration from an object."""
        _store = {key: value for key, value in obj.__dict__.items() if key.isupper()}
        for name, value in _store.items():
            self.config_vars.add(name, value=value)

    def from_dotenv(self, path: str):
        """Load configuration from specified .env path."""
        for name, value in dotenv_to_dict(path).items():
            self.config_vars.add(name, value=value)

    def from_envar(self, name: str, rename: str = None, default: str = None):
        """Set configuration from an environment variable."""
        if not os.getenv(name) and rename is None:
            warn_msg = f"Environment variable '{name}' not found."
            warnings.warn(warn_msg, RuntimeWarning)
            return
        self.config_vars.add(name, alias=rename, default=default, use_env=True)



# -*- coding: utf-8 -*-
import os
from typing import Any, Dict

BOOLEAN_STATES = {"true": True, "false": False, "True": True, "False": False}


def is_boolean_state(value: str) -> bool:
    """Check if a string is 'boolean like'."""
    return value in BOOLEAN_STATES


def is_integer(value: str) -> bool:
    """Check if a value is an integer."""
    try:
        int(value)
        return True
    except ValueError:
        return False


def is_float(value: str) -> bool:
    """Check if a value is a float."""
    if is_integer(value):
        return False
    try:
        float(value)
        return True
    except ValueError:
        return False


def get_integer(value: str) -> int:
    """Get a integer from a value."""
    if is_integer(value):
        return int(value)
    else:
        raise ValueError(f"Not a valid integer. Must be {type(int)}")


def get_float(value: str) -> float:
    """Get a float from a value."""
    if is_float(value):
        return float(value)
    else:
        raise ValueError(f"Not a valid number. Must be {type(float)}")


def get_boolean(value: str) -> bool:
    """Get a boolean from a value."""
    if is_boolean_state(value):
        return BOOLEAN_STATES[value]
    else:
        raise ValueError(f"Not a valid boolean. Must be {type(float)}")


def str_to_py(value: str):
    """Convert an string value to a native python type."""
    rv: Any
    if is_boolean_state(value):
        rv = get_boolean(value)
    elif is_integer(value):
        rv = get_integer(value)
    elif is_float(value):
        rv = get_float(value)
    else:
        rv = value
    return rv


def dotenv_to_dict(path: str) -> Dict[str, Any]:
    """Convert a .env file to a dict."""
    with open(path, "r") as dotenvfile:
        lines = dotenvfile.readlines()

    rv: Dict[str, Any] = dict()
    for line in lines:
        if line.startswith("#"):
            continue
        name, value = line.strip().split("=", 1)
        py_value = str_to_py(value)
        rv[name] = py_value
    return rv


def get_envar(name: str) -> Any:
    rv = None
    try:
        env_value = os.environ[name]
        rv = str_to_py(env_value)
    except KeyError:
        pass
    return rv


def envar_exists(name: str) -> bool:
    return os.getenv(name) is not None
